# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-01-12 03:48

import django.contrib.postgres.fields.jsonb
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


forward = """
    DROP TABLE node_query_cache;

    CREATE TABLE node_query_cache (
        id serial PRIMARY KEY,
        project_id integer REFERENCES project (id) ON DELETE CASCADE,
        orientation integer DEFAULT 0 NOT NULL,
        depth real NOT NULL,
        json_data jsonb,
        json_text_data text,
        msgpack_data bytea
    );

    CREATE INDEX node_query_cache_project_orientation_z_idx ON node_query_cache (project_id, orientation, depth);
"""

backward = """
    DROP TABLE node_query_cache;

    CREATE TABLE node_query_cache (
        project_id integer REFERENCES project (id),
        orientation integer DEFAULT 0 NOT NULL,
        depth real NOT NULL,
        json_data jsonb,
        json_text_data text,
        msgpack_data bytea
    );

    CREATE INDEX node_query_cache_project_orientation_z_idx ON node_query_cache (project_id, orientation, depth);
"""

class Migration(migrations.Migration):
    """Update the node_query_cache project foreign key to support cascading
    deletes.
    """

    dependencies = [
        ('catmaid', '0032_add_simple_section_cache_table'),
    ]

    operations = [
        migrations.RunSQL(forward, backward, [
            migrations.CreateModel(
                name='NodeQueryCache',
                fields=[
                    ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                    ('orientation', models.IntegerField(default=0)),
                    ('depth', models.FloatField(null=True)),
                    ('json_data', django.contrib.postgres.fields.jsonb.JSONField(blank=True, null=True)),
                    ('json_text_data', models.TextField(blank=True, null=True)),
                    ('msgpack_data', models.BinaryField(null=True)),
                    ('project', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='catmaid.Project')),
                ],
                options={
                    'db_table': 'node_query_cache',
                },
            ),
        ])
    ]

