import catmaid.fields
import django.contrib.postgres.functions
from django.db import migrations


class Migration(migrations.Migration):
    """Update the default values for most timestamp fields so that the
    database's transaction start time is used (CURRENT_TIMESTAMP or now()). This
    doesn't include any actual database change, and can be run as "passive"
    state_operations with a noop migration. In fact, tring to apply this as
    regular operation list, crashes the migrate management command. This bug has
    been reported to Django (#31206).
    """

    dependencies = [
        ('catmaid', '0099_make_concept_ids_64_bit'),
    ]

    operations = [
        migrations.RunSQL(migrations.RunSQL.noop, migrations.RunSQL.noop, [
            migrations.AlterField(
                model_name='cardinalityrestriction',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='cardinalityrestriction',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='changerequest',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='changerequest',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='class',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='class',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='classclass',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='classclass',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='classinstance',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='classinstance',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='classinstanceclassinstance',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='classinstanceclassinstance',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='concept',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='concept',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='connector',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='connector',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='connectorclassinstance',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='connectorclassinstance',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='datasource',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='datasource',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='dirtynodegridcachecell',
                name='invalidation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='imagedata',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='imagedata',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='location',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='location',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='log',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='log',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='nblastconfig',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='nblastconfig',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='nblastsample',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='nblastsample',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='nblastsimilarity',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='nblastsimilarity',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='nodegridcachecell',
                name='update_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='point',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='point',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='pointclassinstance',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='pointclassinstance',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='pointcloud',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='pointcloud',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='pointconnector',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='pointconnector',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='pointset',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='pointset',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='regionofinterest',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='regionofinterest',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='regionofinterestclassinstance',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='regionofinterestclassinstance',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='relation',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='relation',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='relationinstance',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='relationinstance',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='restriction',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='restriction',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='review',
                name='review_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='sampler',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='sampler',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='samplerconnector',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='samplerconnector',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='samplerdomain',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='samplerdomain',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='samplerinterval',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='samplerinterval',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='skeletonorigin',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='skeletonorigin',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='stackclassinstance',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='stackclassinstance',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='stackgroupclassinstance',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='stackgroupclassinstance',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='suppressedvirtualtreenode',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='suppressedvirtualtreenode',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='treenode',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='treenode',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='treenodeclassinstance',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='treenodeclassinstance',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='treenodeconnector',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='treenodeconnector',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='volume',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='volume',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='volumeclassinstance',
                name='creation_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
            migrations.AlterField(
                model_name='volumeclassinstance',
                name='edition_time',
                field=catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow),
            ),
        ]),
    ]
