import catmaid.fields
from django.conf import settings
import django.contrib.postgres.functions
from django.db import migrations, models
import django.db.models.deletion


forward = """
    CREATE TABLE catmaid_favorite_project (
        id int GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY,
        user_id int REFERENCES auth_user(id) ON DELETE CASCADE NOT NULL,
        project_id int REFERENCES project(id) ON DELETE CASCADE NOT NULL,
        creation_time timestamptz NOT NULL DEFAULT now(),
        edition_time timestamptz NOT NULL DEFAULT now(),

        rank real NOT NULL,

        txid bigint DEFAULT txid_current()
    );

    -- Trigger
    CREATE TRIGGER on_edit_favorite_project BEFORE UPDATE ON catmaid_favorite_project
        FOR EACH ROW EXECUTE PROCEDURE on_edit();

    -- History
    SELECT create_history_table('catmaid_favorite_project'::regclass, 'edition_time', 'txid');

    -- Indices
    CREATE INDEX catmaid_favorite_project_user_id_idx ON catmaid_favorite_project (user_id) INCLUDE (project_id);
"""


backward = """
    SELECT drop_history_table('catmaid_favorite_project'::regclass);

    DROP TABLE catmaid_favorite_project;
"""


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('catmaid', '0110_add_project_token_model'),
    ]

    operations = [
        migrations.RunSQL(forward, backward, [
            migrations.CreateModel(
                name='FavoriteProject',
                fields=[
                    ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                    ('creation_time', catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow)),
                    ('edition_time', catmaid.fields.DbDefaultDateTimeField(default=django.contrib.postgres.functions.TransactionNow)),
                    ('rank', models.FloatField(default=0)),
                    ('project', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to='catmaid.Project')),
                    ('user', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to=settings.AUTH_USER_MODEL)),
                ],
                options={
                    'db_table': 'catmaid_favorite_project',
                },
            ),
        ]),
    ]
